#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "arduino.h"
#include "request.h"
#include "pilot.h"
#include <math.h>

type_state_t state = WAIT;

void send_state(){
	request_set_runmode(NONE9,0,0);
}

void request_set_runmode(run_state_t st, int speed, int dist){//この関数を実行すると
	command_data_t command_data;
	
	command_data.val[0] = (((st == STR) ? 1 : (st == ROT) ? 2  : (st == LNT) ? 3  : (st == LNT_RS) ? 4  : (st == ARM) ? 5  : (st == ARM_PID) ? 6:0)&0x0f) | ((state << 4)&0xf0);
	command_data.val[1] = speed;
	command_data.val[2] = dist;
	//printf("%x--",command_data.val[0]);
	
	arduino_send(command_data);
}

run_state_t get_runmode(void){
	run_state_t r_st = STP;
	int a,b;
	request_get_runmode(&r_st,&a,&b);
	return r_st;
}

int request_get_runmode(run_state_t *st, int *speed, int *dist){
	command_data_t command_data;
	
	command_data.val[0] = 10 | ((state << 4)&0xf0);
	command_data.val[1] = 0;
	command_data.val[2] = 0;
	
	arduino_clear();
	arduino_send(command_data);
	
	command_data.val[0] = 66;
	usleep(50 * 1000);
	
	if(arduino_receive(&command_data) == 0){
		//printf("com: %d  ",command_data.val[0]);
		*st = (run_state_t)command_data.val[0];//((command_data.val[0] == 1) ? STR : (command_data.val[0] == 2) ? ROT : (command_data.val[0] == 3) ? LNT : (command_data.val[0] == 4) ? LNT_RS : (command_data.val[0] == 5) ? ARM : (command_data.val[0] == 6) ? ARM_PID : STP);
		*speed = command_data.val[1];
		*dist  = command_data.val[2];
		return 0;
	}else{
		return -1;
	}
}

int request_set_camF(int isfront){
	command_data_t command_data;
	
	command_data.val[0] = 8 | ((state << 4)&0xf0);
	if(isfront== 1){ 
		command_data.val[1] = 1;
	}else{
		command_data.val[1] = 2;
	}
	command_data.val[2] = 0;
	
	arduino_clear();
	arduino_send(command_data);
	usleep(50 * 1000);
	return 0;
}

int request_get_armdist(double *dist ,double *height){
	command_data_t command_data;
	
	command_data.val[0] = 7 | ((state << 4)&0xf0);
	command_data.val[1] = 0;
	command_data.val[2] = 0;
	
	arduino_clear();
	arduino_send(command_data);
	usleep(50 * 1000);
	
	if(arduino_receive(&command_data) == 0){
		double h = command_data.val[1];//[mm]
		*dist = sqrt(pow(510,2)-pow(h-686,2));
		*height = h;
		return 0;
	}else{
		return -1;
	}
}

int request_get_dist(int *dist_l, int *dist_r){
	command_data_t command_data;
	
	command_data.val[0] = 11 | ((state << 4)&0xf0);
	command_data.val[1] = 0;
	command_data.val[2] = 0;
	
	arduino_clear();
	arduino_send(command_data);
	usleep(50 * 1000);
	
	if(arduino_receive(&command_data) == 0){
		*dist_l = command_data.val[0];
		*dist_r = command_data.val[1];
		return 0;
	}else{
		return -1;
	}
}

int request_get_batt(double *volt){
	command_data_t command_data;
	
	command_data.val[0] = 14 | ((state << 4)&0xf0);
	command_data.val[1] = 0;
	command_data.val[2] = 0;
	
	arduino_clear();
	arduino_send(command_data);
	usleep(50 * 1000);
	
	if(arduino_receive(&command_data) == 0){
		*volt = command_data.val[0] / 100.0;
		return 0;
	}else{
		return -1;
	}
}
